<?php

namespace SGalinski\SgMail\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgMail\Service\BackendService;
use TYPO3\CMS\Extbase\Persistence\Generic\Query;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;

/**
 * Repository for the Mail object
 */
class MailRepository extends AbstractRepository {
	const SENT = '1';
	const NOT_SENT = '2';

	/**
	 * Returns all mails that are still not sent ordered by priority.
	 *
	 * @param int $limit
	 * @return array|QueryResultInterface
	 */
	public function findMailsToSend($limit = NULL) {
		$query = $this->createQuery();
		if ($limit) {
			$query->setLimit($limit);
		}

		$query->setOrderings(['priority' => Query::ORDER_DESCENDING]);

		return $query->matching($query->equals('sent', FALSE))->execute();
	}

	/**
	 * Find all mails (sent & unsent) for extension key and template name
	 *
	 * @param string $extensionKey
	 * @param string $templateName
	 * @param int $siteroot
	 * @param int $limit
	 * @param array $filters
	 * @return array|QueryResultInterface
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\InvalidQueryException
	 */
	public function findAllEntries($extensionKey, $templateName, $siteroot = 0, $limit = NULL, array $filters = []) {
		$query = $this->createQuery();
		if ($limit) {
			$query->setLimit($limit);
		}

		if ($filters['filterTemplate'] && $filters['filterTemplate'] !== 0
			&& $filters['filterTemplate'] !== '0' && $filters['filterTemplate'] !== ''
		) {
			$constraintsAnd[] = $query->equals('template_name', $filters['filterTemplate']);
		}

		if ($filters['filterLanguage'] && $filters['filterLanguage'] !== 0
			&& $filters['filterLanguage'] !== '0' && $filters['filterLanguage'] !== ''
		) {
			$constraintsAnd[] = $query->equals('language', $filters['filterLanguage']);
		}

		$constraintsOr = [];
		if ($filters['filterFields'] && trim($filters['filterFields'] !== '')) {
			$search = '%' . $filters['filterSearch'] . '%';
			foreach ($filters['filterFields'] as $field) {
				switch ($field) {
					case BackendService::SENDER_FILTER_OPTION :
						$constraintsOr[] = $query->like('from_address', $search);
						break;
					case BackendService::RECIPIENT_FILTER_OPTION :
						$constraintsOr[] = $query->like('to_address', $search);
						break;
					case BackendService::SUBJECT_FILTER_OPTION :
						$constraintsOr[] = $query->like('mail_subject', $search);
						break;
					case BackendService::MAILTEXT_FILTER_OPTION :
						$constraintsOr[] = $query->like('mail_body', $search);
						break;
					case BackendService::CC_FILTER_OPTION :
						$constraintsOr[] = $query->like('cc_addresses', $search);
						break;
					case BackendService::BCC_FILTER_OPTION :
						$constraintsOr[] = $query->like('bcc_addresses', $search);
						break;
				}
			}
			$constraintsAnd[] = $query->logicalOr($constraintsOr);
		}

		if ($filters['sent']) {
			switch ($filters['sent']) {
				case self::SENT:
					$constraintsAnd[] = $query->equals('sent', TRUE);
					break;
				case self::NOT_SENT:
					$constraintsAnd[] = $query->equals('sent', FALSE);
					break;
			}
		}

		$constraintsAnd[] = $query->equals('extension_key', $extensionKey);
		$constraintsAnd[] = $query->equals('site_root_id', (int) $siteroot);

		$query->setOrderings(['tstamp' => Query::ORDER_DESCENDING]);
		return $query->matching($query->logicalAnd($constraintsAnd))->execute(TRUE);
	}
}
